;------------------------------------------------------------------------------
; BDOS-Schnittstelle
;------------------------------------------------------------------------------

;		org	0d000h

		jp	cmd_bios
aBios:		db "BIOS    "
		db 0
		jp	cmd_bdos
aBdos:		db "BDOS    "
		db 0
		jp	cmd_dir
aDir:		db "DIR     "
		db 0
		db    0

;------------------------------------------------------------------------------
; Testen der BIOS-Funktionen
; Aufruf: BIOS fktnr p1
;------------------------------------------------------------------------------

cmd_bios:	rst	20h
		db 19h				; PARH2, 1 Byte hexadezimal aus Kommandozeile in Register E holen
		ret	c			; bei Fehler
		ld	a, 19			; max. BIOS-Funktionsnummer
		cp	e
		ld	a, 3
		ret	c			; bei Fehler (Funktionsnummer > 19)
		ld	l, e
		rst	20h
		db 1Dh				; PARH4, 1 Adresse hexadezimal aus Kommandozeile in DE holen
		ret	c			; bei Fehler
		ld	c, e			; bc := p1
		ld	b, d
		xor	a
		add	a, l
		add	a, l
		add	a, l
		ld	l, a
		ld	h, 0			; hl := funktion*3
		ld	de, bios_tabelle	; Beginn BIOS-Tabelle im RAM
		add	hl, de
;
		rst	20h
		db 5h				; PCRLF, Ausgabe Zeilenschaltung (ODH, OAH)
		ret	c			; bei Fehler
		push	hl
		ld	hl, cmd_bdos1
		ex	(sp), hl
		ld	a, (sys_bank)
		push	af
		push	hl
		ld	hl, sys_rst20ret	; 0042h, freier Bereich fr Systemerweiterungen
		ex	(sp), hl
		ld	a, 0FFh
		jp	sys_jphl		; JMP (HL) in Bank (A)

;------------------------------------------------------------------------------
; Testen der BDOS-Funktionen
; Aufruf: BDOS rufnr p1
;------------------------------------------------------------------------------

cmd_bdos:	rst	20h
		db 19h				; PARH2, 1 Byte hexadezimal aus Kommandozeile in Register E holen
		ret	c			; bei Fehler
		ld	c, e			; c := rufnr
		rst	20h
		db 1Dh				; PARH4, 1 Adresse hexadezimal aus Kommandozeile in DE holen
		ret	c			; bei Fehler
		rst	20h
		db 5h				; PCRLF, Ausgabe Zeilenschaltung (ODH, OAH)
		ret	c			; bei Fehler
;		
		rst	8			; BDOS-Aufruf mit c=Rufnummer und de=bergabeparameter
cmd_bdos1:	ret	c			; bei Fehler
;Ausgabe der Rckkehrparameter A und HL
		rst	20h
		db 6h				; PRNBY, Ausgabe Register A hexadezimal (2 Zeichen)
		ret	c			; bei Fehler
		rst	20h
		db 2h				; PRNSP, Ausgabe ein Leerzeichen
		ret	c			; bei Fehler
		rst	20h
		db 7h				; PRNAD, Ausgabe Register HL hexadezimal (4 Zeichen)
		ret

;------------------------------------------------------------------------------
; 
; Aufruf: DIR [maske]
;------------------------------------------------------------------------------

cmd_dir:	ld	de, 80h
		ld	c, 26
		rst	8			; DMA-Adresse auf 80h setzen
		ret	c			; bei Fehler
		ld	hl, fcb1+1
		ld	a, (hl)
		cp	' '
		jr	nz, cmd_dir2		; wenn ein Parameter folgt
;
		ld	b, 11			; sonst mit '???????????' initialisieren
cmd_dir1:	ld	(hl), '?'
		inc	hl
		djnz	cmd_dir1
cmd_dir2:	ld	de, fcb1
		ld	c, 11h			
		rst	8			; ersten Dir-Eintrag suchen
		ret	c			; bei Fehler
		cp	0FFh			
		jr	z, cmd_dir8		; wenn nichts gefunden wurde
		ld	b, 1
cmd_dir3:	add	a, a			; A = Directory-Code
		add	a, a
		add	a, a
		add	a, a
		add	a, a
		ld	e, a			; E := A*32
		ld	d, 0
		ld	ix, 80h
		add	ix, de			; ix := gltiger Verzeichniseintrag im Puffer
		bit	7, (ix+0Ah)
		jr	nz, cmd_dir6
		djnz	cmd_dir5
		ld	b, 2
		call	sub_bd_crlf
		ret	c			; bei Fehler
		ld	a, (fcb1)
		dec	a
		cp	0FFh
		jr	nz, cmd_dir4
		ld	c, 19h
		rst	8
		ret	c			; bei Fehler
cmd_dir4:	add	a, 41h
		call	sub_bd_outa
		ret	c			; bei Fehler
		ld	c, 20h
		ld	e, 0FFh
		rst	8			; Benutzernummer holen
		ret	c			; bei Fehler
		add	a, 30h
		call	sub_bd_outa
		ret	c			; bei Fehler
cmd_dir5:	push	bc
		call	sub_diranz
		pop	bc
		jr	c, cmd_dir7
cmd_dir6:	call	sub_bd_status
		ret	c			; bei Fehler
		ret	nz
		ld	c, 12h
		rst	8			; nchsten eintrag holen
		ret	c			; bei Fehler
		cp	0FFh
		ret	z			; wenn kein weiterer Eintrag gefunden wurde
		jr	cmd_dir3
cmd_dir7:	inc	sp
		inc	sp
		ret
;
cmd_dir8:	ld	de, aNoFile		; "\r\nNo File$"
cmd_dir9:	ld	a, (de)
		inc	de
		cp	'$'
		ret	z
		call	sub_bd_outa
		ret	c			; bei Fehler
		jr	cmd_dir9
		ret

aNoFile:	db 0Dh	
		db 0Ah
		db "No File$"

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------


sub_diranz:	ld	a, ':'
		call	sub_bd_outa
		ret	c			; bei Fehler
		ld	a, ' '
		call	sub_bd_outa
		ret	c			; bei Fehler
		ld	b, 8
sub_diranz1:	inc	ix
		ld	a, (ix+0)
		and	7Fh
		call	sub_bd_outa
		ret	c			; bei Fehler
		djnz	sub_diranz1
		ld	a, ' '
		call	sub_bd_outa
		ret	c			; bei Fehler
		ld	b, 3
sub_diranz2:	inc	ix
		ld	a, (ix+0)
		and	7Fh
		call	sub_bd_outa
		ret	c			; bei Fehler
		djnz	sub_diranz2
		bit	7, (ix-2)
		ld	a, '*'
		jr	nz, sub_diranz3
		ld	a, ' '
sub_diranz3:	call	sub_bd_outa
		ret

;------------------------------------------------------------------------------
; Ausgabe CR LF
;------------------------------------------------------------------------------

sub_bd_crlf:	ld	a, 0Dh
		call	sub_bd_outa
		ret	c			; bei Fehler
		ld	a, 0Ah

;------------------------------------------------------------------------------
; Ausgabe Zeichen A
;------------------------------------------------------------------------------

sub_bd_outa:	push	bc
		push	de
		ld	e, a
		ld	c, 2
		rst	8			; Konsolenausgabe 
		pop	de
		pop	bc
		ret

;------------------------------------------------------------------------------
; Konsole bereit?
;------------------------------------------------------------------------------

sub_bd_status:	ld	c, 0Bh
		rst	8			; Konsolenstatus holen
		ret	c			; bei Fehler
		or	a
		ret	z
		ld	a, 0
		scf
		ret
