;-----------------------------------------------------------------------------
; BO1 Systemanlauf
; E000, Bank 0 - hier startet alles
;-----------------------------------------------------------------------------

;		org 	0E000h

		jp	cmd_init
		db "#       "	; Initialisierung nach RESET
		db 0
		jp	cmd_z9001
		db "###     "
		db 0
		db 0

fkt_E019:	jp	fkt_searchhi
fkt_e01c:	jp	fkt_up_r3
fkt_e01f:	jp	fkt_up_w1
fkt_e022:	jp	fkt_up_w2

;------------------------------------------------------------------------------
;Einsprung nach	Reset, Bank 00 ist selektiert
;------------------------------------------------------------------------------

cmd_init:
		ld	sp, 200h		; Stackpointer auf 200h setzen
		ld	a, 2
		ld	i, a			; Interrupt-Register auf 0200h setzen
		ei
		ld	a, 0			; Bank 00 merken
		ld	(sys_bank), a
		ld	(user_bank), a
;
		ld	de, asc_cmd_bo2		; "##      "
		ld	bc, 0DF00h
		ld	l, 0FFh
		call	fkt_searchhi		; suche	BO-Startkommando
		jr	nc, cmd_z9001		; wenn nicht gefunden
		ld	(user_bank), a		; sonst gefundene Bank selektieren
		ld	(sys_bank), a
		out	(0FFh),	a
		ex	de, hl
		jp	(hl)			; und Programm starten

asc_cmd_bo2:	db "##      "			; Kennung BO-Initialisierung

;------------------------------------------------------------------------------
; Bank 55 selektieren und Sprung nach Z9001-BIOS-WBOOT.
;------------------------------------------------------------------------------

cmd_z9001:	ld	a, standard_bank	; Bank 55 selektieren
		ld	hl, sys_setbank
		ld	(hl), 0D3h 		; OUT FF
		inc	hl
		ld	(hl), 0FFh
		inc	hl
		ld	(hl), 0E9h 		; JP (HL)
		ld	hl, WBOOT		; Warmstart, Sprung ins Z9001-BIOS WBOOT
		jp	sys_setbank

;------------------------------------------------------------------------------
; fkt_E019 Kommandosuche im Hi-Bank-Bereich (C000-DFFF)
; gesucht wird von BC (L) bis C000h (00h).
; BC: Startadresse, DE: Suchstring, L: Bank
; Return: A: Bank, DE: Adr
;------------------------------------------------------------------------------

fkt_searchhi:	ld	a, b
		cp	0C0h
		ccf
		ret	nc			; zurck, wenn BC < 0C000h
;
		ld	a, l			; Bank aus L
		and	0F0h			; Hi-Teil
		ld	h, a			; merken
		ld	a, (sys_bank)		; Systembank holen
		and	0Fh                     ; Systembank (Lo)
		or	h                       ; kombinieren
;
fkt_searchhi1:	ld	(sys_bank), a		; Bank merken
		out	(0FFh),	a		; und switchen
		ld	h, 0C0h			; Suche von BC bis 0C000h
		call	sub_search		; Suchen
		ld	a, (sys_bank)		; Systembank holen
		scf
		jr	z, fkt_searchhi2	; wenn gefunden
		ld	h, a			; Systembank merken
		and	0F0h			; Hi-Teil
		jr	z, fkt_searchhi2	; wenn Suche bis Bank 0x beendet
;
		ld	a, l			; Suchbank (Hi) verringern
		sub	10h
		ld	l, a
		ld	a, h			; aktuelle Systembank
		sub	10h			; (Hi) verringern
		ld	bc, 0DF00h		; Suche ab 0DF00h abwrts
		jr	fkt_searchhi1		; weitersuchen
;		
fkt_searchhi2:	push	af
		ld	a, (user_bank)		; Bank restaurieren
		ld	(sys_bank), a
		out	(0FFh),	a
		pop	af
		ret

;------------------------------------------------------------------------------
; Kommandosuche mit '?'-Joker (in einer Bank)
; es wird abwrts von (BC) bis (HL) in aktueller Bank auf xx00-Adressen gesucht
; BC: Suchadresse_von, HL: Suchadresse_bis, DE: Suchstring
; Return: A = 0ffh, Cy=1 nicht gefunden
; A=0, DE=Adr wenn gefunden
;------------------------------------------------------------------------------

sub_search:	ld	a, b
		cp	h			; Bereichsende erreicht?
		jr	nc, sub_search1		; weitersuchen
		xor	a			; A = 0
		dec	a			; A = 0ffh, Cy=1
		ret				; nicht gefunden
;		
sub_search1:	ld	a, (bc)			; ist auf der Adresse BC ein JMP-Befehl?
		cp	0C3h 			; JMP
		jr	nz, sub_search7		; nein -> nchsten Bereich
; folgt ein 0-Byte hinter JMP und Kdo-Name?
		push	hl
		ld	hl, 11			; 3 Byte JMP + 8 Byte Name
		add	hl, bc			; HL = Adr. nchster Tabelleneintrag
		ld	a, (hl)			; 
		pop	hl
		or	a			; 0-Byte?
		jr	nz, sub_search7		; nein -> nchster Bereich
;
		push	hl
		push	bc
		push	de
		ld	hl, 3
		add	hl, bc			; HL = BC+3 = adr Kdo-Name
;
		ld	bc, fcb2+1
		ld	a, 8			; 8 Zeichen
sub_search2:	push	af
		ld	a, (hl)			; Buchstaben holen
		ld	(bc), a			; in Param2 kopieren
		ld	a, (de)			; Suchstring-Buchstaben 
		cp	'?'			; Joker-Zeichen?
		jr	z, sub_search4		; dann ok
		or	a
		jr	nz, sub_search3		; 0-Bytes in Leerzeichen wandeln
		ld	a, ' '
sub_search3:	cp	(hl)			; Zeichen vergleichen
		jr	nz, sub_search5		; wenn ungleich
;		
sub_search4:	inc	hl			; nchstes Zeichen Kdo-Tabelle
		inc	de			; nchstes Zeichen Suchstring
		inc	bc			; nchstes Zeichen Param2
		pop	af
		dec	a			; alle 8 Zeichen bearbeitet?
		jr	nz, sub_search2		; nein -> weiter vergleichen
;Kdo-Namen gefunden
		pop	de			; Suchstring restaurieren
		pop	bc			; aktuelle Suchadresse
		inc	bc
		ld	a, (bc)
		ld	e, a
		inc	bc
		ld	a, (bc)
		ld	d, a			; DE = JMP-Adresse des Kdo
		dec	bc
		dec	bc			; BC zurcksetzen
		xor	a			; A=0
		jr	sub_search6
;
sub_search5:	pop	de			; Namen nicht gefunden
		pop	de
		pop	bc
sub_search6:	ld	hl, 12			; Lnge kompletter Tabelleneintrag
		add	hl, bc
		ld	c, l
		ld	b, h			; BC = BC+12 (Adr nchster KDO-Eintrag)
		pop	hl
		ret	z			; zurck, wenn Namen gefunden
		jr	sub_search		; sonst weitersuchen
;		
sub_search7:	dec	b			; nchsten Bereich 
		ld	c, 0			; xx00h-Adresse
		jr	sub_search		; weitersuchen

;-----------------------------------------------------------------------------
; fkt_e01c (UP R3) 3 Byte von (HL)(A) in E, D, H lesen
; A: Bank HL: Adresse IX: Startadresse
;-----------------------------------------------------------------------------

fkt_up_r3:	and	0F0h			; Bank (Hi)
		ld	d, a			; Bank (Hi) merken
		ld	a, (sys_bank)		; Systembank holen
		and	0Fh			; Systembank (Lo)
		or	d			; kombinieren
		out	(0FFh),	a		; und switchen
		ld	e, (hl)			; 3 Bytes holen
		inc	hl
		ld	d, (hl)
		inc	hl
		ld	h, (hl)
		ld	a, (sys_bank)
		out	(0FFh),	a		; auf Systembank schalten
		jp	(ix)			; Sprung nach (IX)

;-----------------------------------------------------------------------------
; fkt_e01f (UP W1) 1 Byte aus E auf (HL)(A) schreiben
; A: Bank E: Byte HL: Adresse
;-----------------------------------------------------------------------------

fkt_up_w1:	and	0F0h			; Bank (Hi)
		ld	c, a			; Bank (Hi) merken
		ld	a, (sys_bank)		; Systembank holen
		and	0Fh			; Systembank (Lo)
		or	c			; kombinieren
		out	(0FFh),	a		; und switchen
		ld	(hl), e			; E nach (HL) schreiben
		ld	a, (sys_bank)
		out	(0FFh),	a		; auf Systembank schalten
		jp	(ix)			; Sprung nach (IX)

;-----------------------------------------------------------------------------
; fkt_e022 (UP W2) 2 Byte aus E, D auf (HL)(A) schreiben
; A: Bank DE: Byte HL: Adresse
;-----------------------------------------------------------------------------

fkt_up_w2:	and	0F0h			; Bank (Hi)
		ld	c, a			; Bank (Hi) merken
		ld	a, (sys_bank)		; Systembank holen
		and	0Fh			; Systembank (Lo)
		or	c			; kombinieren
		out	(0FFh),	a		; und switchen
		ld	(hl), e			; E nach (HL) schreiben
		inc	hl
		ld	(hl), d			; D nach (HL) schreiben
		dec	hl			; HL zurcksetzen
		ld	a, (sys_bank)
		out	(0FFh),	a		; auf Systembank schalten
		jp	(ix)			; Sprung nach (IX)

