;------------------------------------------------------------------------------
; DBIOS
;------------------------------------------------------------------------------

;Aufbau Speicherbank E000(xE):
; 0E000h
; 0E001h
; 0E002h-0E005h
; 0E013h + Offset (up_write)
; aus fdt.inc:
; 0E013h		; Standard-Kommando-Tabelle fr Modul FDK, Bank xE
; 0E540h		; Standard-Kommando-Tabelle fr Modul FDT, Bank xE
;
; 0E006h	alv0
; 0E038h  	alv1
; 0E06Ah  	alv2
; 0E09Ch  	alv3
; 0E0CEh  	alv4
; 0E100h  	csv0
; 0E120h  	csv1
; 0E140h  	csv2
; 0E160h  	csv3
; 0E180h  	dirbuf0
;
; 0E413h	DPHtab		DPH-Tabelle fr max 16 Eintrge, 5 belegt
; 0E433h	u_dpbf0	DPBF0
; 0E43Dh	u_dpbf1	DPBF1
; 0E447h	u_dpbf2	DPBF2
; 0E451h	u_dpbf3	DPBF3
; 0E45Bh	u_dpbf4	DPBF4
; 0E465h 	in RDT		FCB
; 0E536h	unk_0_CA7E	Umschaltung Bank (A), LDIR, Bank Sysbank
; 0E580h-0E5FFh in RDT		DMA-Puffer
;
; Doppelbelegung? berpatcht alv0 - Bereich!
; 0E00Bh	unk_0_C988	Tabelle mit 4 Eintrgen, diese verweisen auf je 5 Byte
;
;
; Aufbau Speicherbank E000(FF)

; 0E55Bh	u_dph0	DPH0
; 0E57Ch	u_dph1	DPH1
; 0E59Dh	u_dph2	DPH2
; 0E5BEh	u_dph3	DPH3
; 0E5DFh	u_dph4	DPH4
; 0E56Dh	u_dpb0	DPB0
; 0E58Eh	u_dpb1	DPB1
; 0E5AFh	u_dpb2	DPB2
; 0E5D0h	u_dpb3	DPB3
; 0E5F1h	u_dpb4	DPB4
;
;------------------------------------------------------------------------------

;		org	0c800h

		jp	cmd_d_dbios
aDbios:		db "$DBIOS  "
		db 0
		jp	cmd_p_dbios
aDbios_0:	db "%DBIOS  "
		db 0
		jp	cmd_rdt
aRdt:		db "RDT     "
		db 0
		db    0

;------------------------------------------------------------------------------
; Kommando Read Disk Track
; RDT lw spur sektor
; Funktion: Ausfhrung einer Disketten-Leseoperation ber die BIOS-Schnittstelle
; (READ TEST). Die Parameter lw, spur und Sektor entsprechen den bei den BIOS-
; Funktionen SELDSK, SETTRK und SETSEC im Register C bergebenen Parametern. Sie
; mssen hier als Hexadezimalzahl eingegeben werden.
; 	lw 	: 0=A 1=B 2=C
; 	spur 	: logische Spurnummer ab 0
; 	Sektor 	: logische Sektornummer ab 1
; Der Inhalt der gelesenen Sektors (128 Byte) wird am Bildschirm angezeigt. Mit
; diesem Kommando kann schnell ein Blick auf eine Diskette geworfen werden.
;------------------------------------------------------------------------------

cmd_rdt:	call	sub_0_CAB1	; Bank xE nach sys_bank laden und aktivieren
		ld	b, 6
		ld	iy, 0E465h	; FCB
		call	sub_0_CABE	; Parameter nach (iy+3), (iy+4), ... holen
		ret	c		; bei Fehler
		ld	hl, 0E580h
		ld	(iy+6),	l
		ld	(iy+7),	h
		ld	(iy+8),	0Eh
		call	up_read
		ret	c		; bei Fehler
		or	a
		jr	nz, cmd_rdt1
		ld	hl, 0E580h	; DMA-Buffer
		ld	de, 0E5FFh
		rst	20h
		db 0Bh			; DISPO, Anzeige eines Speicherbereiches hexadezimal und ASCII von (HL) bis (DE)
		ret
cmd_rdt1:
		rst	20h
		db 6h			; PRNBY, Ausgabe Register A hexadezimal (2 Zeichen)
		ret

;------------------------------------------------------------------------------
; Kaltstart von $DBIOS
;------------------------------------------------------------------------------

cmd_d_dbios:	ld	e, 0FFh		; E000h(E) := 0FFh
		ld	hl, 0E000h
		ld	a, 0Eh
		rst	20h
		db  42h 		; W1   , 1 Byte aus E auf (HL) in Bank (A) schreiben
		ret	c		; bei Fehler

; DBIOS-UP eintragen
; 88 HOME  DBIOS BIOS-Ruf 8 Rcksetzen Diskettensystem
; 89 SELDS DBIOS BIOS-Ruf 9 Auswahl Laufwerk
; 8A SETTR DBIOS BIOS-Ruf 10 Auswahl Spur
; 8B SETSE DBIOS BIOS-Ruf 11 Auswahl Sektor
; 8C SETDM DBIOS BIOS-Ruf 12 Festlegen DMA-Adresse, die Bank wird auf 0FFH gesetzt
; 8D READ  DBIOS BIOS-Ruf 13 Lesen eines 128 Byte Sektors von Diskette auf die DMA-Adresse
; 8E WRITE DBIOS BIOS-Ruf 14 Schreiben eines 128 Byte Sektors out Diskette
; 90 SECT  DBIOS BIOS-Ruf 16 Sektortranslate
; 94 SETDB DBIOS Festlegen der Bank zur DMA-Adresse

		ld	hl, unk_0_C96D	; Tabelle der Kommandos
		ld	b, 9		; Anzahl
		ld	a, (sys_bank)
loc_0_C864:	ld	e, (hl)		; DE = Adresse
		inc	hl
		ld	d, (hl)
		inc	hl
		ld	c, (hl)		; C = UP-Nr.
		inc	hl
		rst	20h
		db 0FFh			; EXTUP, Einbindung eines weiteren Unterprogrammes in das System UP
		ret	c		; bei Fehler
		djnz	loc_0_C864

; Kopieren der Disk-Parameter-Tabellen u.a. in den Upper-RAM
		ld	a, (sys_bank)
		ld	b, a
		ld	hl, DPHtab
		ld	de, 0E413h
		ld	c, 0Eh		; bank xE
		ld	ix, 20h
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
; DPHx und DPBx kommen in Bank FF
		ld	hl, u_dph0
		ld	de, 0E55Bh
		ld	c, 0FFh		; Bank FF
		ld	ix, 12h
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dph1
		ld	de, 0E57Ch
		ld	ix, 12h
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dph2
		ld	de, 0E59Dh
		ld	ix, 12h
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dph3
		ld	de, 0E5BEh
		ld	ix, 12h
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dph4
		ld	de, 0E5DFh
		ld	ix, 12h
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
;
		ld	hl, u_dpb0
		ld	de, 0E56Dh
		ld	ix, 0Fh
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dpb1
		ld	de, 0E58Eh
		ld	ix, 0Fh
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dpb2
		ld	de, 0E5AFh
		ld	ix, 0Fh
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dpb3
		ld	de, 0E5D0h
		ld	ix, 0Fh
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dpb4
		ld	de, 0E5F1h
		ld	ix, 0Fh
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
; DBPFx komen in Bank xE
		ld	hl, u_dpbf0
		ld	de, 0E433h
		ld	c, 0Eh		; Bank xE
		ld	ix, 0Ah
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dpbf1
		ld	de, 0E43Dh
		ld	ix, 0Ah
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dpbf2
		ld	de, 0E447h
		ld	ix, 0Ah
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dpbf3
		ld	de, 0E451h
		ld	ix, 0Ah
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ld	hl, u_dpbf4
		ld	de, 0E45Bh
		ld	ix, 0Ah
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
;
		ld	hl, unk_0_C988
		ld	de, 0E00Bh
		ld	c, 0Eh
		ld	ix, 8
		RST 	20H
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
;
		ret

;------------------------------------------------------------------------------
; Warmstart von %DBIOS
;------------------------------------------------------------------------------

cmd_p_dbios:
		; Kopieren des UP "Umschaltung Bank (A), LDIR, Bank Sysbank" nach Bank 0E536h (xE)
		ld	hl, unk_0_CA7E	; UP "Umschaltung Bank (A), LDIR, Bank Sysbank"
		ld	de, 0E536h
		ld	a, (sys_bank)	; aktuelle Bank
		ld	b, a
		ld	c, 0Eh
		ld	ix, 0Ah
		rst	20h
		db 44h			; LDIRU, Blocktransport zwischen Bnken von (HL) (B) nach (DE) (C), Lnge = IX
		ret	c		; bei Fehler
		ret

; Tabelle der DBIOS-UPs. Die UPs werden in Systemkomponente BD eingebunden
unk_0_C96D:     dw up_home
                db 88h
                dw up_selds
                db 89h
                dw up_settr
                db 8Ah
                dw up_setse
                db 8Bh
                dw up_setdm
                db 8Ch
                dw up_read
                db 8Dh
                dw up_write
                db 8Eh
                dw up_sect
                db 90h
                dw up_setdb
                db 94h

; wird nach 0E00Bh (E) transportiert
; vp: Bedeutung unklar
unk_0_C988:     dw unk_0_CA88
                dw unk_0_CA88
                dw unk_0_CA8D
                dw unk_0_CA92

; wird nach 0E413h (E) transportiert
DPHtab:     	dw 0E55Bh               ; DPH0
                dw 0E57Ch               ; DPH1
                dw 0E59Dh               ; DPH2
                dw 0E5BEh               ; DPH3
                dw 0E5DFh               ; DPH4
                dw 0
                dw 0
                dw 0
                dw 0
                dw 0
                dw 0
                dw 0
                dw 0
                dw 0
                dw 0
                dw 0

; DPH0 wird nach 0E55Bh (F) transportiert
u_dph0:    	dw 0
                dw 0
                dw 0
                dw 0
                dw dirbuf0
                dw 0E56Dh		;dpb0
                dw csv0			;0E100h
                dw alv0			;0E006h
                dw 0E433h		;dpbf0

; DPH1 wird nach 0E57Ch (F) transportiert
u_dph1:    	dw 0
                dw 0
                dw 0
                dw 0
                dw dirbuf0
                dw 0E58Eh		;dpb1
                dw csv1			;0E120h
                dw alv1			;0E038h
                dw 0E43Dh		;dpbf1

; DPH2 wird nach 0E59Dh (F) transportiert
u_dph2:     	dw 0
                dw 0
                dw 0
                dw 0
                dw dirbuf0
                dw 0E5AFh		;dpb2
                dw csv2			;0E140h
                dw alv2			;0E06Ah
                dw 0E447h		;dpbf2

; DPH3 wird nach 0E5BEh (F) transportiert
u_dph3:     	dw 0
                dw 0
                dw 0
                dw 0
                dw dirbuf0
                dw 0E5D0h		;dpb3
                dw csv3			;0E160h
                dw alv3			;0E09Ch
                dw 0E451h		;dpbf3

; DPH4 wird nach 0E5DFh (F) transportiert
u_dph4:     	dw 0
                dw 0
                dw 0
                dw 0
                dw dirbuf0
                dw 0E5F1h		;dpb4
                dw csv3			;0E160h
                dw alv4			;0E0CEh
                dw 0E45Bh		;dpbf4

; wird nach 0E56Dh (F) transportiert
u_dpb0:		dw  0028h		; SPT 40 Sektoren/Spur
		db    4			; BSH Blockgre 2K
		db  0Fh			; BLM
		db    0			; EXM
		dw  0185h		; DSM 780K-Diskette
		dw  007Fh		; DRM 128 Dir-Eintrge
		db  0C0h		; AL0
		db    0			; AL1
		dw  0020h		; CKS
		dw  0004h		; OFF 4 Systemblcke

; wird nach 0E433h (E) transportiert
u_dpbf0:	db  20h			; DPBFL
		db    3			; DPBSL Sektor 1024 Byte
		db    7			; DPBBM 8 Sektoren im internen DBIOS-1K-Buffer
		db    1			; DPBST 1 Stepimpuls pro Spur
		db    0			; DPBSN keine Sektorverschiebung auf Rckseite
		db 0A0h			; DPBTR 160 log. Spuren
		db    0			; DPBDN log. Laufwerksnummer 0
		db  38h			; DPBGP Lnge GAP 3
		dw	unk_0_CA97	; DPBTT Adr. Sektortranslatetabelle

; DPB1 wird nach 0E58Eh (F) transportiert
u_dpb1:		db  28h 		; SPT 40 Sektoren/Spur
		db    0
		db    4			; BSH Blockgre 2K
		db  0Fh			; BLM
		db    0			; EXM
		db  85h			; DSM 780K-Diskette
		db    1
		db  7Fh			; DRM 128 Dir-Eintrge
		db    0
		db 0C0h			; AL0
		db    0			; AL1
		db  20h			; CKS
		db    0
		db    4			; OFF 4 Systemblcke
		db    0

; DPBF1 wird nach 0E43Dh (E) transportiert
u_dpbf1:	db  20h			; DPBFL
		db    3			; DPBSL Sektor 1024 Byte
		db    7			; DPBBM 8 Sektoren im internen DBIOS-1K-Buffer
		db    1			; DPBST 1 Stepimpuls pro Spur
		db    0			; DPBSN keine Sektorverschiebung auf Rckseite
		db 0A0h			; DPBTR 160 log. Spuren
		db    1			; DPBDN log. Laufwerksnummer 0
		db  38h			; DPBGP Lnge GAP 3
		dw	unk_0_CA97	; DPBTT Adr. Sektortranslatetabelle

; DPB2 wird nach 0E5AFh (F) transportiert
u_dpb2:		db  20h			; SPT 32 Sektoren/Spur
		db    0
		db    4			; BSH Blockgre 2K
		db  0Fh			; BLM
		db    1			; EXM
		db  49h			; DSM 320K-Diskette
		db    0
		db  3Fh			; DRM 64 Dir-Eintrge
		db    0
		db  80h			; AL0
		db    0			; AL1
		db  10h			; CKS
		db    0
		db    3			; OFF 3 Systemblcke
		db    0

; DPBF2 wird nach 0E447h (E) transportiert
u_dpbf2:	db    0			; DPBFL
		db    1			; DPBSL Sektor 1024 Byte
		db    7			; DPBBM 8 Sektoren im internen DBIOS-1K-Buffer
		db    1			; DPBST 1 Stepimpuls pro Spur
		db    0			; DPBSN keine Sektorverschiebung auf Rckseite
		db  28h			; DPBTR 160 log. Spuren
		db    2			; DPBDN log. Laufwerksnummer 0
		db  20h			; DPBGP Lnge GAP 3
		dw	unk_0_CA97	; DPBTT Adr. Sektortranslatetabelle

; DPB3 wird nach 0E5D0h (F) transportiert
u_dpb3:		db  28h			; SPT 40 Sektoren/Spur
		db    0
		db    4			; BSH Blockgre 2K
		db  0Fh			; BLM
		db    0			; EXM
		db  8Fh			; DSM 800K-Diskette
		db    1
		db 0BFh			; DRM 192 Dir-Eintrge
		db    0
		db 0E0h			; AL0
		db    0			; AL1
		db  20h			; CKS
		db    0
		db    0			; OFF 0 Systemblcke
		db    0

; DPBF3 wird nach 0E451h (E) transportiert
u_dpbf3:	db  20h			; DPBFL
		db    3			; DPBSL Sektor 1024 Byte
		db    7			; DPBBM 8 Sektoren im internen DBIOS-1K-Buffer
		db    1			; DPBST 1 Stepimpuls pro Spur
		db    0			; DPBSN keine Sektorverschiebung auf Rckseite
		db 0A0h			; DPBTR 160 log. Spuren
		db    0			; DPBDN log. Laufwerksnummer 0
		db  38h			; DPBGP Lnge GAP 3
		dw	unk_0_CA97	; DPBTT Adr. Sektortranslatetabelle

; DPB4 wird nach 0E5F1h (F) transportiert
u_dpb4:		db    0			; SPT 8000h Sektoren/Spur
		db  80h
		db    3			; BSH Blockgre 1K
		db    7			; BLM
		db    0			; EXM
		db 0FFh			; DSM 1MB-Diskette
		db    0
		db  3Fh			; DRM 64 Dir-Eintrge
		db    0
		db 0C0h			; AL0
		db    0			; AL1
		db  10h			; CKS
		db    0
		db    0			; OFF 0 Systemblcke
		db    0

; DPBF4 wird nach 0E45Bh (E) transportiert
u_dpbf4:	db  80h ; 

;vp: Speicher doppelt belegt? berpatcht? Hier msste wohl DPBF4 weitergehen, da dieser 10 Byte lang sein sollte
;wird von cmd_p_dbios nach 0E536h (xE) kopiert
unk_0_CA7E:     out     (0FFh), a
                ldir
                ld      a, (sys_bank)
                out     (0FFh), a
                ret

;---------
; s. 0E00Bh	unk_0_C988	Tabelle mit 4 Eintrgen, diese verweisen auf je 5 Byte
; Bedeutung unklar; wird nirgends verwendet, ??? phys. Laufwerksparameter ???
unk_0_CA88:	db  39h ; 
		db  50h ; 80 Spuren Vorderseite
		db  50h ; 80 Spuren Rckseite
		db 0E1h ; 
		db    9
unk_0_CA8D:	db  18h
		db  28h ; 40 Spuren Vorderseite
		db    0 ; 0 Spuren Rckseite
		db  98h ; 
		db  15h
unk_0_CA92:	db    0
		db  4Dh ; 77 Spuren Vorderseite
		db    0 ; 0 Spuren Rckseite
		db  6Fh ; 
		db  29h ; 

; Sektortranslatetabelle
unk_0_CA97:	db    1
		db    2
		db    3
		db    4
		db    5
		db    6
		db    7
		db    8
		db    9
		db  0Ah
		db  0Bh
		db  0Ch
		db  0Dh
		db  0Eh
		db  0Fh
		db  10h
		db  11h
		db  12h
		db  13h
		db  14h
		db  15h
		db  16h
		db  17h
		db  18h
		db  19h
		db  1Ah


;------------------------------------------------------------------------------
; UPs zu RDT
;------------------------------------------------------------------------------

; Bank xE nach sys_bank laden und aktivieren
sub_0_CAB1:	ld	a, (sys_bank)
		and	0F0h
loc_0_CAB6:	or	0Eh
		ld	(sys_bank), a
		out	(0FFh),	a
		ret

; Parameter nach (iy+3), (iy+4), ... holen
sub_0_CABE:	push	iy
		push	de
loc_0_CAC1:	rst	20h
		db 19h			; PARH2, 1 Byte hexadezimal aus Kommandozeile in Register E holen
		jr	c, loc_0_CAD3
		ld	(iy+3),	e
		inc	iy
		ld	a, (fcb1+1)
		cp	20h		; folgt noch ein Parameter ?
		jr	z, loc_0_CAD3	; nein -> Ende
		djnz	loc_0_CAC1
loc_0_CAD3:	pop	de
		pop	iy
		ret

;------------------------------------------------------------------------------
; 89 SELDS, BIOS-Ruf 9 Auswahl Laufwerk
;------------------------------------------------------------------------------

up_selds:	call	sub_0_CD00	; lokal Bank xE aktivieren
		call	sub_0_CD24
		ret	c		; bei Fehler
		ret	z
		ld	(iy+3),	c
		bit	0, e
		ret	nz
		bit	7, (ix+0)
		ret	nz
		push	hl
		ld	a, c
		ld	hl, 0E000h
		cp	(hl)
		jr	nz, up_selds1
		ld	(hl), 0FFh
		ld	hl, 0E001h
		res	0, (hl)
up_selds1:	nop
		nop
		or	a
		pop	hl
		ret

;------------------------------------------------------------------------------
; 88 HOME , BIOS-Ruf 8 Rcksetzen Diskettensystem
;------------------------------------------------------------------------------

up_home:	call	sub_0_CD00	; lokal Bank xE aktivieren
		ld	hl, 0E001h
		bit	0, (hl)
		jr	nz, up_home1
		ld	a, 0FFh
		ld	(0E000h), a
up_home1:	ld	(iy+4),	0
		or	a
		ret

;------------------------------------------------------------------------------
; 8A SETTR, BIOS-Ruf 10 Auswahl Spur
;------------------------------------------------------------------------------

up_settr:	call	sub_0_CD00	; lokal Bank xE aktivieren
		ld	(iy+4),	c
		or	a
		ret

;------------------------------------------------------------------------------
; 8B SETSE, BIOS-Ruf 11 Auswahl Sektor
;------------------------------------------------------------------------------

up_setse:	call	sub_0_CD00	; lokal Bank xE aktivieren
		ld	(iy+5),	c
		or	a
		ret

;------------------------------------------------------------------------------
; 8C SETDM, BIOS-Ruf 12 Festlegen DMA-Adresse, die Bank wird auf 0FFH gesetzt
;------------------------------------------------------------------------------

up_setdm:	call	sub_0_CD00	; lokal Bank xE aktivieren
		ld	(iy+6),	c
		ld	(iy+7),	b
		ld	(iy+8),	0FFh
		or	a
		ret

;------------------------------------------------------------------------------
; 90 SECT , BIOS-Ruf 16 Sektortranslate
;------------------------------------------------------------------------------

up_sect:	ld	h, b
		ld	l, c
		inc	hl
		or	a
		ret

;------------------------------------------------------------------------------
; 94 SETDB, Festlegen der Bank zur DMA-Adresse
;------------------------------------------------------------------------------

up_setdb:	call	sub_0_CD00	; lokal Bank xE aktivieren
		ld	(iy+8),	c
		or	a
		ret

;------------------------------------------------------------------------------
; 8D READ , BIOS-Ruf 13 Lesen eines 128 Byte Sektors von Diskette auf die DMA-Adresse
;------------------------------------------------------------------------------

up_read:	call	sub_0_CD00	; lokal Bank xE aktivieren
		ld	(iy+9),	0
		ld	c, (iy+3)
		call	sub_0_CD24
		ret	c		; bei Fehler
		ret	z
		bit	7, (ix+0)
		jp	nz, loc_0_CE08
		ld	(iy+0Bh), 46h
		jr	up_write1

;------------------------------------------------------------------------------
; 8E WRITE, BIOS-Ruf 14 Schreiben eines 128 Byte Sektors out Diskette
;------------------------------------------------------------------------------

up_write:	call	sub_0_CD00	; lokal Bank xE aktivieren
		ld	(iy+9),	c
		ld	c, (iy+3)
		call	sub_0_CD24
		ret	c		; bei Fehler
		ret	z
		bit	7, (ix+0)
		jp	nz, loc_0_CE0B
		ld	(iy+0Bh), 45h
;
up_write1:	ld	a, (iy+5)
		or	a
		ld	a, 1
		ret	z
		call	sub_0_CD4C
		ret	c		; bei Fehler
		ld	a, (iy+5)
		dec	a
		cp	l
		ld	a, 1
		ret	nc
		ld	a, (ix+0)
		and	3
		ld	d, a
		ld	a, (iy+4)
		cp	d
		ld	a, (ix+1)
		ld	b, (ix+7)
		jr	nc, up_write2
		ld	l, 1Ah
		xor	a
		ld	b, 7
		res	6, (iy+0Bh)
up_write2:	ld	(iy+10h), a
		ld	(iy+12h), b
		ld	(iy+13h), 0FFh
		or	a
		jr	nz, up_write3
		ld	(iy+13h), 80h
up_write3:	ld	b, (iy+5)
		dec	b
		call	sub_0_CD6B
		ld	a, e
		dec	a
		add	a, d
		ld	(iy+11h), a
		call	sub_0_CDA5
		di
		ld	a, (0E000h)
		inc	a
		jr	z, loc_0_CC41
		ld	hl, 0E002h
		push	iy
		ld	b, 4
up_write4:	ld	a, (hl)
		cp	(iy+0Ch)
		jr	nz, up_write5
		inc	hl
		inc	iy
		djnz	up_write4
up_write5:	pop	iy
		jr	nz, loc_0_CC30
up_write6:	ld	a, (iy+5)
		dec	a
		and	(ix+2)
		ld	b, a
		rra
		ld	d, a
		ld	e, 0
		rr	e
		ld	hl, 0E013h
		add	hl, de
		ld	e, (iy+6)
		ld	d, (iy+7)
		ld	c, (iy+8)
		bit	0, (iy+0Bh)
		jr	z, up_write7
		ld	a, (0E001h)
		set	0, a
		ld	(0E001h), a
		ex	de, hl
up_write7:	push	bc
		ld	a, (iy+7)
		call	sub_0_CC5F
		pop	bc
		bit	0, (iy+0Bh)
		jr	z, loc_0_CC25
		ld	a, b
		cp	(ix+2)
		jr	z, loc_0_CC28
		ld	a, (iy+9)
		cp	1
		jr	z, loc_0_CC28
loc_0_CC25:	xor	a
		jr	loc_0_CC5D
loc_0_CC28:	ld	bc, 0
		call	sub_0_CCC8
		jr	loc_0_CC5D
loc_0_CC30:	ld	a, (0E001h)
		bit	0, a
		jr	z, loc_0_CC41
		ld	bc, 0Ah
		call	sub_0_CCC8
		jr	c, loc_0_CC5D
		jr	nz, loc_0_CC5D
loc_0_CC41:	ld	a, (iy+9)
		cp	2
		ld	d, (iy+0Bh)
		push	de
		call	sub_0_CCA5
		pop	de
		ld	(iy+0Bh), d
		jr	c, loc_0_CC5D
		jr	nz, loc_0_CC5D
		ld	a, (iy+3)
		ld	(0E000h), a
		jr	up_write6
loc_0_CC5D:	ei
		ret
sub_0_CC5F:	and	0F8h
		cp	0E0h
		jr	z, loc_0_CC71
		ld	a, c
		and	0F0h
		or	0Eh
		ld	bc, 80h
		di
		jp	0E536h		; Umschaltung Bank (A), LDIR, Bank Sysbank
;
loc_0_CC71:	ld	a, c
		and	0Fh
		ld	c, a
		ld	a, (sys_bank)
		and	0F0h
		or	c
		push	de
		ex	(sp), ix
		ld	e, a
		ld	a, (sys_bank)
		ld	d, a
		bit	0, (iy+0Bh)
		jr	z, loc_0_CC8B
		ld	d, e
		ld	e, a
loc_0_CC8B:	ld	c, 0FFh
		ld	b, 80h
		di
loc_0_CC90:	out	(c), d
		ld	a, (hl)
		inc	hl
		out	(c), e
		ld	(ix+0),	a
		inc	ix
		djnz	loc_0_CC90
		ld	a, (sys_bank)
		out	(0FFh),	a
		pop	ix
		ret

sub_0_CCA5:	ld	e, (ix+3)
		push	iy
		ex	(sp), ix
		ld	bc, 0Ah
		add	ix, bc
		ld	d, 4
loc_0_CCB3:	ld	bc, 0Ah
		ld	hl, 0E001h
		call	fdt_prd
		jr	nc, loc_0_CCFC
		dec	d
		jr	z, loc_0_CCEA
		call	fdt_morec
		jr	c, loc_0_CCEA
		jr	loc_0_CCB3
sub_0_CCC8:	push	iy
		ex	(sp), ix
		ld	de, 0Ah
		add	ix, de
		add	ix, bc
		ld	d, 4
loc_0_CCD5:	ld	bc, 0Ah
		ld	hl, 0E001h
		call	fdt_pwr
		jr	nc, loc_0_CCFC
		dec	d
		jr	z, loc_0_CCEA
		call	fdt_morec
		jr	c, loc_0_CCEA
		jr	loc_0_CCD5
loc_0_CCEA:	ei
		push	af
		ld	a, 0FFh
		ld	(0E000h), a
		pop	af
		call	fdt_error
		pop	ix
		ret	c		; bei Fehler
		ld	a, 1
		or	a
		ret
loc_0_CCFC:	pop	ix
		xor	a
		ret

;------------------------------------------------------------------------------
; ein CALL sub_0_CD00 startet das nachfolgende Programm in Bank xE und schaltet
; anschlieend die Bank zurck
;------------------------------------------------------------------------------

sub_0_CD00:	pop	hl		; RET-Adresse des Callers
		push	de
		push	ix

		ld	a, (sys_bank)	; Bank xE aktivieren
		ld	d, a		; orig. Bank
		and	0F0h
		or	0Eh
		ld	(sys_bank), a
		out	(0FFh),	a

		push	de		; orig. Bank sichern

		call	sub_0_CD23	; CALL (HL), d.h. Aufruf des nach CALL sub_0_CD00 folgenden Codes

		ld	d, a		; Rckgabewert sichern
		ex	(sp), hl	; hl = orig. Bank
		ld	a, h		; orig. Bank
		pop	hl		; Stack bereinigen (?)
		ld	(sys_bank), a
		out	(0FFh),	a	; orig. Bank wiederherstellen
		ld	a, d		; Rckgabewert
		pop	ix
		pop	de
		ret
;
sub_0_CD23:	jp	(hl)

;------------------------------------------------------------------------------
; UP Laufswerksauswahl
; in 	C: DPH-Nummer (0..15)
; out	HL: Adr. DPH-Block
;	IX: Adr. DPBF-Blocks
;------------------------------------------------------------------------------

sub_0_CD24:	ld	hl, 0
		ld	a, c
		cp	10h		; < 16 ?
		jr	nc, loc_0_CD47	; Fehler
		push	de
		ld	hl, 0E413h	; DPH-Tabelle fr max 16 Eintrge
		ld	e, c
		ld	d, 0
		add	hl, de		; 2 x DPH-Nummer dazuaddieren
		add	hl, de
		ld	e, (hl)
		inc	hl
		ld	d, (hl)		; DE := Adr. des DPH-Blocks
		push	de
		ld	hl, 10h		; Offset 16 Byte
		add	hl, de		; Addresse des DPBF-Blocks
		ld	a, 0FFh		; DPH liegt in Bank FF
		rst	20h
		db 41h			; R3   , 3 Byte von (HL) (A) in E, D, H lesen
		push	de
		pop	ix		; IX = Adr. DPBF-Blocks
		pop	hl		; HL = Adr. des DPH-Blocks
		pop	de
		ret	c		; bei Fehler
loc_0_CD47:	ld	a, l
		or	h
		ld	a, 1
		ret

;------------------------------------------------------------------------------
; UP zu write
;------------------------------------------------------------------------------

sub_0_CD4C:	ld	de, 0Ah
		add	hl, de
		ld	a, 0FFh
		rst	20h
		db 41h			; R3   , 3 Byte von (HL) (A) in E, D, H lesen
		ret	c		; bei Fehler
		ld	hl, 0
		add	hl, de
		ld	a, 0FFh
		rst	20h
		db 40h			; R1   , 1 Byte von (HL) Bank (A) in A lesen
		ret	c		; bei Fehler
		ld	l, a
		push	hl
		ld	hl, 0Dh
		add	hl, de
		ld	a, 0FFh
		rst	20h
		db 40h			; R1   , 1 Byte von (HL) Bank (A) in A lesen
		pop	hl
		ld	h, a
		ret

;------------------------------------------------------------------------------
; UP zu write
;------------------------------------------------------------------------------

sub_0_CD6B:	push	bc
		ld	a, b
		ld	d, (ix+2)
		and	d
		xor	b
loc_0_CD72:	srl	a
		rr	d
		jr	nz, loc_0_CD72
		ld	b, a
		ld	a, (iy+10h)
		ld	d, (ix+2)
		inc	d
		or	a
		jr	z, loc_0_CD88
loc_0_CD83:	rrc	d
		dec	a
		jr	nz, loc_0_CD83
loc_0_CD88:	ld	a, b
		or	a
		jr	z, loc_0_CD90
		xor	a
loc_0_CD8D:	add	a, d
		djnz	loc_0_CD8D
loc_0_CD90:	ld	e, a
		ld	b, (iy+10h)
		ld	a, b
		or	a
		ld	a, l
		jr	z, loc_0_CD9D
loc_0_CD99:	srl	a
		djnz	loc_0_CD99
loc_0_CD9D:	sub	e
		cp	d
		jr	nc, loc_0_CDA2
		ld	d, a
loc_0_CDA2:	inc	e
		pop	bc
		ret

;------------------------------------------------------------------------------
; UP zu write
;------------------------------------------------------------------------------

sub_0_CDA5:	push	bc
		push	de
		ld	a, (ix+6)
		and	3
		ld	(iy+0Ch), a
		ld	c, e
		dec	c
		ld	d, (iy+4)
		ld	e, d
		bit	4, (ix+0)
		jr	nz, loc_0_CDC7
		bit	5, (ix+0)
		jr	z, loc_0_CDE9
		srl	d
		jr	nc, loc_0_CDE9
		jr	loc_0_CDD9
loc_0_CDC7:	ld	b, (ix+5)
		srl	b
		ld	a, d
		sub	b
		jr	c, loc_0_CDE9
		bit	3, (ix+0)
		jr	nz, loc_0_CDD8
		cpl
		add	a, b
loc_0_CDD8:	ld	d, a
loc_0_CDD9:	set	2, (iy+0Ch)
		ld	a, (ix+4)
		add	a, c
		cp	c
		ld	c, a
		jr	nz, loc_0_CDE9
		ld	a, 1
		jr	loc_0_CDEA
loc_0_CDE9:	xor	a
loc_0_CDEA:	ld	(iy+0Eh), a
		ld	(iy+0Dh), d
		inc	c
		ld	a, e
		cp	h
		jr	c, loc_0_CE02
		push	hl
		ld	l, (ix+8)
		ld	h, (ix+9)
		dec	c
		ld	b, 0
		add	hl, bc
		ld	c, (hl)
		pop	hl
loc_0_CE02:	ld	(iy+0Fh), c
		pop	de
		pop	bc
		ret
loc_0_CE08:	xor	a
		inc	a
		ret
loc_0_CE0B:	xor	a
		ret

;------------------------------------------------------------------------------
; RAM-Bereich DBIOS E000H (E)
;------------------------------------------------------------------------------

		org	0E000h

alv0		equ	0E006h
alv1		equ	0E038h
alv2		equ	0E06Ah
alv3		equ	0E09Ch
alv4		equ	0E0CEh
csv0            equ	0E100h
csv1		equ	0E120h
csv2		equ	0E140h
csv3		equ	0E160h
dirbuf0		equ	0E180h

