;-----------------------------------------------------------------------------
; Arbeitsspeicher 
;-----------------------------------------------------------------------------

IOBYT		equ	0004h		; IO-Byte (in SIO)
LISW		equ	0015h		; Schalter fr Hardcopy (in SIO)

sys_rst20ret	equ	0042h		; RST20-Return-Code
sys_wboot	equ	004Dh		; WBOOT, auf 0000h steht JP 004Dh

fcb1		equ	005Ch
fcb2		equ	006Ch
; Aufbau FCB
; 00	FDRIVE	Laufwerkscode (keins - 0, A - 1, B - 2)
; 01-08	FNAME	Dateiname, mit Leerzeichen aufgefllt
; 09-0B	FTYP	Dateityp, mit Leerzeichen aufgefllt
; 0C	PSUM
; 0D	ARB (?)
; 0E	BLNR
; 0F	LBLNR
; 10
; 11
; 12-13	AADR
; 14-15	EADR
; 16-17	SADR
; 18	SBY

CONBU		equ	0080h		; Eingabepuffer

para_ATTYC	equ	0EFC9h		; Adresse tty-Treiber fr CONST, Beginn Adretabelle fr Gertetreiber (in SIO)

user_bank	equ	0EFF1h		; aktuelle Bank
sys_jphl	equ	0EFF2h		; hier steht der Befehl: JMP (HL) in Bank (A)
sys_setbank	equ	0EFF5h		; hier steht der Befehl zum Umschalten der Bank (OUT A, 0FFh; RET)

sys_isr		equ	0EFF8h		; ISR-Routine
sys_bank	equ	0EFF9h		; aktuelle Bank des Systems

;-----------------------------------------------------------------------------
; I/O
;-----------------------------------------------------------------------------

CTC0:		EQU	80H
CTC2:		EQU	82H
CTC3:		EQU	83H

DPIO1A:		EQU	88H		;PIO1 A DATEN
DPIO1B:		EQU	89H		;PIO1 B DATEN
SPIO1A:		EQU	8AH		;PIO1 A KOMMANDO
SPIO1B:		EQU	8BH		;PIO1 B KOMMANDO

DPIOA:		EQU	90H		;TASTATUR-PIO2 A DATEN
DPIOB:		EQU	91H		;TASTATUR-PIO2 B DATEN
SPIOA:		EQU	92H		;TASTATUR-PIO2 A KOMMANDO
SPIOB:		EQU	93H		;TASTATUR-PIO2 B KOMMANDO

;-----------------------------------------------------------------------------
; Systemaufrufe
;-----------------------------------------------------------------------------

WBOOT		equ	0F003h		; Z9001-BIOS Warmstart (CCP)
;BOS		equ	0F314h		; Z9001-BOS-Start (BO2, CCP)
;RCONB		equ	0F365h		; Eingabe Zeichenkette (UP)

;-----------------------------------------------------------------------------
; RAM-Nutzung
;-----------------------------------------------------------------------------

; 0000-003a	Restart-Adressen
; 0040-005b	Bankrckschaltungsprogramme
; 02c0-02ff	Programmanfnge der Restarts
; eff1-efff	Merkzellen fr Bankumschaltung, Interruptrckkehr

;BO
bios_tabelle	equ	0e200h		; Bank F
; Sichern des Bereichs 100h..1FFh nach E600h..E6FFh(0FFh)
; Tauschen des Bereichs 200h..2FFh mit E700h..E7FFh(0FFh)


;UP
sys_bank_up	equ	0Ch
up_tabelle	equ	0e000h		; Bank C	; UP-Liste in RAM E000(C), jeweils 2 Byte Adr und 1 Byte Bank
;		equ	0e150h				; lst_bios_kdo in RAM, jeweils 2 Byte Adr und 1 Byte Bank
up_biostab	equ	0e200h				; UP-Sprungverteiler

;BDOS
;		equ	0e300h		; Bank C	; BDOS-Tabelle

;TAPE
tape_fcb	equ	0E55Ch		; Bank E	fcb
tape_dma	equ	0E580h		; Bank E


;-----------------------------------------------------------------------------
; Konstanten
;-----------------------------------------------------------------------------

standard_bank	equ	55h

;-----------------------------------------------------------------------------
; AS-Funktionen
;-----------------------------------------------------------------------------

hi              function x,(x>>8)&255
lo              function x, x&255
nextpage	function x, ((x+0ffh)/100h)*100h

